#lang racket/base

(require racket/contract
         racket/match
         unstable/sequence
         fancy-app
         "../base/main.rkt"
         "../util/immutable.rkt"
         "../util/list-pair-contract.rkt"
         "join-list.rkt")

(module+ test
  (require rackunit
           "../list/list-ref-take-drop.rkt"))

(provide
 (contract-out
  [lens-join/hash (->* () #:rest (listof2 any/c lens?) (lens/c any/c immutable-hash?))]))


(define (value-list->hash keys vs)
  (make-immutable-hash (map cons keys vs)))

(define (split-slice n vs)
  (define grouped
    (for/list ([group (in-slice n vs)])
      group))
  (define (get-ith i)
    (map (list-ref _ i) grouped))
  (build-list n get-ith))

(module+ test
  (check-equal? (split-slice 3 '(a 1 FOO b 2 BAR c 3 BAZ))
                '((a b c) (1 2 3) (FOO BAR BAZ))))


(define (lens-join/hash . keys/lenses)
  (match-define (list keys lenses) (split-slice 2 keys/lenses))
  (define list-lens (apply lens-join/list lenses))
  (define (get target)
    (value-list->hash keys (lens-view list-lens target)))
  (define (set target new-view-hash)
    (lens-set list-lens target (map (hash-ref new-view-hash _) keys)))
  (make-lens get set))

(module+ test
  (define a-b-lens (lens-join/hash 'b third-lens
                                   'a first-lens))
  (check-equal? (lens-view a-b-lens '(1 2 3))
                (hash 'a 1 'b 3))
  (check-equal? (lens-set a-b-lens '(1 2 3) (hash 'a 100 'b 200))
                '(100 2 200)))

